@@ -11,37 +11,60 @@ from functools import partial
 import pytest
 import numpy as np
 from scipy.io import savemat
-from numpy.testing import assert_array_equal, assert_equal
+from numpy.testing import assert_array_equal, assert_equal, assert_allclose
 
-from mne.channels import (rename_channels, read_ch_connectivity,
-                          find_ch_connectivity, make_1020_channel_selections,
+from mne.channels import (rename_channels, read_ch_adjacency, combine_channels,
+                          find_ch_adjacency, make_1020_channel_selections,
                           read_custom_montage, equalize_channels)
-from mne.channels.channels import (_ch_neighbor_connectivity,
-                                   _compute_ch_connectivity)
+from mne.channels.channels import (_ch_neighbor_adjacency,
+                                   _compute_ch_adjacency)
 from mne.io import (read_info, read_raw_fif, read_raw_ctf, read_raw_bti,
                     read_raw_eeglab, read_raw_kit, RawArray)
 from mne.io.constants import FIFF
 from mne.utils import _TempDir, run_tests_if_main
 from mne import (pick_types, pick_channels, EpochsArray, EvokedArray,
-                 make_ad_hoc_cov, create_info)
+                 make_ad_hoc_cov, create_info, read_events, Epochs)
 from mne.datasets import testing
 
 io_dir = op.join(op.dirname(__file__), '..', '..', 'io')
 base_dir = op.join(io_dir, 'tests', 'data')
 raw_fname = op.join(base_dir, 'test_raw.fif')
+eve_fname = op .join(base_dir, 'test-eve.fif')
 fname_kit_157 = op.join(io_dir, 'kit', 'tests', 'data', 'test.sqd')
 
 
-def test_reorder_channels():
+@pytest.mark.parametrize('preload', (True, False))
+@pytest.mark.parametrize('proj', (True, False))
+def test_reorder_channels(preload, proj):
     """Test reordering of channels."""
-    raw = read_raw_fif(raw_fname, preload=True).crop(0, 0.1)
+    raw = read_raw_fif(raw_fname).crop(0, 0.1).del_proj()
+    if proj:  # a no-op but should test it
+        raw._projector = np.eye(len(raw.ch_names))
+    if preload:
+        raw.load_data()
+    # with .reorder_channels
+    if proj and not preload:
+        with pytest.raises(RuntimeError, match='load data'):
+            raw.copy().reorder_channels(raw.ch_names[::-1])
+        return
     raw_new = raw.copy().reorder_channels(raw.ch_names[::-1])
+    assert raw_new.ch_names == raw.ch_names[::-1]
+    if proj:
+        assert_allclose(raw_new._projector, raw._projector, atol=1e-12)
+    else:
+        assert raw._projector is None
+        assert raw_new._projector is None
     assert_array_equal(raw[:][0], raw_new[:][0][::-1])
     raw_new.reorder_channels(raw_new.ch_names[::-1][1:-1])
     raw.drop_channels(raw.ch_names[:1] + raw.ch_names[-1:])
     assert_array_equal(raw[:][0], raw_new[:][0])
     with pytest.raises(ValueError, match='repeated'):
         raw.reorder_channels(raw.ch_names[:1] + raw.ch_names[:1])
+    # and with .pick
+    reord = [1, 0] + list(range(2, len(raw.ch_names)))
+    rev = np.argsort(reord)
+    raw_new = raw.copy().pick(reord)
+    assert_array_equal(raw[:][0], raw_new[rev][0])
 
 
 def test_rename_channels():
@@ -139,8 +162,8 @@ def test_set_channel_types():
     pytest.raises(ValueError, raw.set_channel_types, ch_types)
 
 
-def test_read_ch_connectivity():
-    """Test reading channel connectivity templates."""
+def test_read_ch_adjacency():
+    """Test reading channel adjacency templates."""
     tempdir = _TempDir()
     a = partial(np.array, dtype='<U7')
     # no pep8
@@ -154,30 +177,30 @@ def test_read_ch_connectivity():
     mat_fname = op.join(tempdir, 'test_mat.mat')
     savemat(mat_fname, mat, oned_as='row')
 
-    ch_connectivity, ch_names = read_ch_connectivity(mat_fname)
-    x = ch_connectivity
+    ch_adjacency, ch_names = read_ch_adjacency(mat_fname)
+    x = ch_adjacency
     assert_equal(x.shape[0], len(ch_names))
     assert_equal(x.shape, (3, 3))
     assert_equal(x[0, 1], False)
     assert_equal(x[0, 2], True)
     assert np.all(x.diagonal())
-    pytest.raises(ValueError, read_ch_connectivity, mat_fname, [0, 3])
-    ch_connectivity, ch_names = read_ch_connectivity(mat_fname, picks=[0, 2])
-    assert_equal(ch_connectivity.shape[0], 2)
+    pytest.raises(ValueError, read_ch_adjacency, mat_fname, [0, 3])
+    ch_adjacency, ch_names = read_ch_adjacency(mat_fname, picks=[0, 2])
+    assert_equal(ch_adjacency.shape[0], 2)
     assert_equal(len(ch_names), 2)
 
     ch_names = ['EEG01', 'EEG02', 'EEG03']
     neighbors = [['EEG02'], ['EEG04'], ['EEG02']]
-    pytest.raises(ValueError, _ch_neighbor_connectivity, ch_names, neighbors)
+    pytest.raises(ValueError, _ch_neighbor_adjacency, ch_names, neighbors)
     neighbors = [['EEG02'], ['EEG01', 'EEG03'], ['EEG 02']]
-    pytest.raises(ValueError, _ch_neighbor_connectivity, ch_names[:2],
+    pytest.raises(ValueError, _ch_neighbor_adjacency, ch_names[:2],
                   neighbors)
     neighbors = [['EEG02'], 'EEG01', ['EEG 02']]
-    pytest.raises(ValueError, _ch_neighbor_connectivity, ch_names, neighbors)
-    connectivity, ch_names = read_ch_connectivity('neuromag306mag')
-    assert_equal(connectivity.shape, (102, 102))
+    pytest.raises(ValueError, _ch_neighbor_adjacency, ch_names, neighbors)
+    adjacency, ch_names = read_ch_adjacency('neuromag306mag')
+    assert_equal(adjacency.shape, (102, 102))
     assert_equal(len(ch_names), 102)
-    pytest.raises(ValueError, read_ch_connectivity, 'bananas!')
+    pytest.raises(ValueError, read_ch_adjacency, 'bananas!')
 
     # In EGI 256, E31 sensor has no neighbour
     a = partial(np.array)
@@ -192,8 +215,8 @@ def test_read_ch_connectivity():
     mat = dict(neighbours=nbh)
     mat_fname = op.join(tempdir, 'test_isolated_mat.mat')
     savemat(mat_fname, mat, oned_as='row')
-    ch_connectivity, ch_names = read_ch_connectivity(mat_fname)
-    x = ch_connectivity.todense()
+    ch_adjacency, ch_names = read_ch_adjacency(mat_fname)
+    x = ch_adjacency.todense()
     assert_equal(x.shape[0], len(ch_names))
     assert_equal(x.shape, (4, 4))
     assert np.all(x.diagonal())
@@ -214,7 +237,7 @@ def test_read_ch_connectivity():
     mat = dict(neighbours=nbh)
     mat_fname = op.join(tempdir, 'test_error_mat.mat')
     savemat(mat_fname, mat, oned_as='row')
-    pytest.raises(ValueError, read_ch_connectivity, mat_fname)
+    pytest.raises(ValueError, read_ch_adjacency, mat_fname)
 
 
 def test_get_set_sensor_positions():
@@ -264,43 +287,43 @@ def test_1020_selection():
 
 
 @testing.requires_testing_data
-def test_find_ch_connectivity():
-    """Test computing the connectivity matrix."""
+def test_find_ch_adjacency():
+    """Test computing the adjacency matrix."""
     data_path = testing.data_path()
 
     raw = read_raw_fif(raw_fname, preload=True)
-    sizes = {'mag': 828, 'grad': 1700, 'eeg': 386}
+    sizes = {'mag': 828, 'grad': 1700, 'eeg': 384}
     nchans = {'mag': 102, 'grad': 204, 'eeg': 60}
     for ch_type in ['mag', 'grad', 'eeg']:
-        conn, ch_names = find_ch_connectivity(raw.info, ch_type)
+        conn, ch_names = find_ch_adjacency(raw.info, ch_type)
         # Silly test for checking the number of neighbors.
         assert_equal(conn.getnnz(), sizes[ch_type])
         assert_equal(len(ch_names), nchans[ch_type])
-    pytest.raises(ValueError, find_ch_connectivity, raw.info, None)
+    pytest.raises(ValueError, find_ch_adjacency, raw.info, None)
 
     # Test computing the conn matrix with gradiometers.
-    conn, ch_names = _compute_ch_connectivity(raw.info, 'grad')
+    conn, ch_names = _compute_ch_adjacency(raw.info, 'grad')
     assert_equal(conn.getnnz(), 2680)
 
     # Test ch_type=None.
     raw.pick_types(meg='mag')
-    find_ch_connectivity(raw.info, None)
+    find_ch_adjacency(raw.info, None)
 
     bti_fname = op.join(data_path, 'BTi', 'erm_HFH', 'c,rfDC')
     bti_config_name = op.join(data_path, 'BTi', 'erm_HFH', 'config')
     raw = read_raw_bti(bti_fname, bti_config_name, None)
-    _, ch_names = find_ch_connectivity(raw.info, 'mag')
+    _, ch_names = find_ch_adjacency(raw.info, 'mag')
     assert 'A1' in ch_names
 
     ctf_fname = op.join(data_path, 'CTF', 'testdata_ctf_short.ds')
     raw = read_raw_ctf(ctf_fname)
-    _, ch_names = find_ch_connectivity(raw.info, 'mag')
+    _, ch_names = find_ch_adjacency(raw.info, 'mag')
     assert 'MLC11' in ch_names
 
-    pytest.raises(ValueError, find_ch_connectivity, raw.info, 'eog')
+    pytest.raises(ValueError, find_ch_adjacency, raw.info, 'eog')
 
     raw_kit = read_raw_kit(fname_kit_157)
-    neighb, ch_names = find_ch_connectivity(raw_kit.info, 'mag')
+    neighb, ch_names = find_ch_adjacency(raw_kit.info, 'mag')
     assert neighb.data.size == 1329
     assert ch_names[0] == 'MEG 001'
 
@@ -360,4 +383,70 @@ def test_equalize_channels():
     assert epochs is epochs2
 
 
+def test_combine_channels():
+    """Test channel combination on Raw, Epochs, and Evoked."""
+    raw = read_raw_fif(raw_fname, preload=True)
+    raw_ch_bad = read_raw_fif(raw_fname, preload=True)
+    raw_ch_bad.info['bads'] = ['MEG 0113', 'MEG 0112']
+    epochs = Epochs(raw, read_events(eve_fname))
+    evoked = epochs.average()
+    good = dict(foo=[0, 1, 3, 4], bar=[5, 2])  # good grad and mag
+
+    # Test good cases
+    combine_channels(raw, good)
+    combined_epochs = combine_channels(epochs, good)
+    assert_array_equal(combined_epochs.events, epochs.events)
+    combine_channels(evoked, good)
+    combine_channels(raw, good, drop_bad=True)
+    combine_channels(raw_ch_bad, good, drop_bad=True)
+
+    # Test with stimulus channels
+    combine_stim = combine_channels(raw, good, keep_stim=True)
+    target_nchan = len(good) + len(pick_types(raw.info, meg=False, stim=True))
+    assert combine_stim.info['nchan'] == target_nchan
+
+    # Test results with one ROI
+    good_single = dict(foo=[0, 1, 3, 4])  # good grad
+    combined_mean = combine_channels(raw, good_single, method='mean')
+    combined_median = combine_channels(raw, good_single, method='median')
+    combined_std = combine_channels(raw, good_single, method='std')
+    foo_mean = np.mean(raw.get_data()[good_single['foo']], axis=0)
+    foo_median = np.median(raw.get_data()[good_single['foo']], axis=0)
+    foo_std = np.std(raw.get_data()[good_single['foo']], axis=0)
+    assert_array_equal(combined_mean.get_data(),
+                       np.expand_dims(foo_mean, axis=0))
+    assert_array_equal(combined_median.get_data(),
+                       np.expand_dims(foo_median, axis=0))
+    assert_array_equal(combined_std.get_data(),
+                       np.expand_dims(foo_std, axis=0))
+
+    # Test bad cases
+    bad1 = dict(foo=[0, 376], bar=[5, 2])  # out of bounds
+    bad2 = dict(foo=[0, 2], bar=[5, 2])  # type mix in same group
+    with pytest.raises(ValueError, match='"method" must be a callable, or'):
+        combine_channels(raw, good, method='bad_method')
+    with pytest.raises(TypeError, match='"keep_stim" must be of type bool'):
+        combine_channels(raw, good, keep_stim='bad_type')
+    with pytest.raises(TypeError, match='"drop_bad" must be of type bool'):
+        combine_channels(raw, good, drop_bad='bad_type')
+    with pytest.raises(ValueError, match='Some channel indices are out of'):
+        combine_channels(raw, bad1)
+    with pytest.raises(ValueError, match='Cannot combine sensors of diff'):
+        combine_channels(raw, bad2)
+
+    # Test warnings
+    raw_no_stim = read_raw_fif(raw_fname, preload=True)
+    raw_no_stim.pick_types(meg=True, stim=False)
+    warn1 = dict(foo=[375, 375], bar=[5, 2])  # same channel in same group
+    warn2 = dict(foo=[375], bar=[5, 2])  # one channel (last channel)
+    warn3 = dict(foo=[0, 4], bar=[5, 2])  # one good channel left
+    with pytest.warns(RuntimeWarning, match='Could not find stimulus'):
+        combine_channels(raw_no_stim, good, keep_stim=True)
+    with pytest.warns(RuntimeWarning, match='Less than 2 channels') as record:
+        combine_channels(raw, warn1)
+        combine_channels(raw, warn2)
+        combine_channels(raw_ch_bad, warn3, drop_bad=True)
+    assert len(record) == 3
+
+
 run_tests_if_main()
